<?php

set_include_path(implode(PATH_SEPARATOR, array(
    dirname(__FILE__) . '/system/classes/dbtables',
    dirname(__FILE__) . '/system/classes/mappers',
    dirname(__FILE__) . '/system/classes/models',
    dirname(__FILE__) . '/system/classes/api',
    get_include_path()
)));

class Youtubefeed extends Tools_Plugins_Abstract
{

    private $_pageUrl = null;
    private $_googleApiKey;

    const ACT_INDEX = 'index';
    const ACT_GETTHUMBS = 'getthumbs';
    const ACT_GETPLAYER = 'getplayer';
    const ACT_SETTINGS = 'showsettings';
    const ACT_APPLY = 'applysettings';
    const MODE_PLAYER = 'player';
    const MODE_THUMBS = 'thumbs';
    const MODE_PLAYLIST = 'playlist';
    const MODE_CHANNEL = 'channels';
    const URL_PLAYLIST_FEED = 'http://gdata.youtube.com/feeds/api/playlists/';
    const YOUTUBEFEED_SECURE_TOKEN = 'YoutubefeedToken';

    public function  __construct($options, $seotoasterData)
    {
        parent::__construct($options, $seotoasterData);

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');

        $this->_googleApiKey = $configHelper->getConfig('googleApiKey');
        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        if (($scriptPaths = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) !== false) {
            $this->_view->setScriptPath($scriptPaths);
        }
        $this->_view->addScriptPath(__DIR__ . '/views/');

    }

    public function run($requestedParams = array())
    {
        $this->_requestedParams = $requestedParams;
        $dispatchersResult = parent::run($requestedParams);
        $this->_view->AdminLogged = Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT);
        $this->_view->pageId = $this->_seotoasterData['id'];

        $this->_view->random = (end($this->_options) == 'random') ? '1' : '0';
        if (isset($this->_options[0])) {
            $this->_view->mode = $this->_options[0];
            if (isset($this->_options[1])) {
                $this->_view->request = $this->_options[1];
            }
            $mapper = YoutubefeedMapper::getInstance();
            $config = $mapper->selectProperties($this->_seotoasterData['id']);
            if (!empty($config)) {
                $this->_view->usewithVideolink = $config[0]->getUseVideolink();
            } else {
                $this->_view->usewithVideolink = '0';
            }
            $this->_view->translator = $this->_translator;
            return $this->_view->render('utubvi.phtml');
        }


    }

    public function _getVideoThumbs()
    {
        $start = $this->_request->getParam('start');
        $pageId = $this->_request->getParam('pageId');
        $random = $this->_request->getParam('random');
        $mapper = YoutubefeedMapper::getInstance();
        $config = $mapper->selectProperties($pageId);
        $requestParam = trim($this->_request->getParam('requestStr'));
        $requestType = $this->_request->getParam('requestType');

        $realMaxResult = 5;
        if (!empty($config)) {
            $maxResults = $config[0]->getThumbPerPage();
            $realMaxResult = $maxResults;
            $this->_view->thumbsWidth = $config[0]->getThumpWidth();
            $this->_view->thumbsHeight = $config[0]->getThumbHeight();
        } else {
            $maxResults = 5;
            $this->_view->thumbsWidth = 180;
            $this->_view->thumbsHeight = 200;
        }
        if ($random == '1') {
            $maxResults = 50;
        }
        $pageToken = $start;
        $queryParams = array(
            'q' => $requestParam,
            'maxResults' => $maxResults,
            'order' => 'relevance',
            'part' => 'id,snippet',
        );

        $next = '';
        $prev = '';
        $youtubeData = $this->_processFeedData($requestType, $requestParam, $queryParams, $pageToken);
        $videos = $youtubeData['videos'];
        $videoFeed = $youtubeData['feed'];

        if (isset($videoFeed['info']['nextPageToken'])) {
            $next = $videoFeed['info']['nextPageToken'];
        }
        if (isset($videoFeed['info']['prevPageToken'])) {
            $prev = $videoFeed['info']['prevPageToken'];
        }

        if ($random == '1') {
            shuffle($videos);
            if (count($videos) > $realMaxResult) {
                $videos = array_slice($videos, 0, $realMaxResult);
            }
        }

        $this->_view->thumbs = $videos;

        $this->_view->next = $next;
        $this->_view->prev = $prev;

        return $this->_view->render('thumbs.phtml');

    }

    protected function _processFeedData($requestType, $requestParam, $queryParams, $pageToken)
    {
        $videos = array();
        $videoFeed = array();
        $youtube = new Youtube(array('key' => $this->_googleApiKey));
        if (self::MODE_THUMBS == $requestType) {
            $queryParams['type'] = 'video';
            $videoFeed = $youtube->paginateResults($queryParams, $pageToken);
        } elseif (self::MODE_PLAYLIST == $requestType) {
            $videoFeed = $youtube->getPlaylistItemsByPlaylistId($requestParam, $queryParams['maxResults'], $pageToken);
        } elseif (self::MODE_CHANNEL == $requestType) {
            $queryParams['q'] = '';
            $queryParams['channelId'] = $requestParam;
            $queryParams['type'] = 'video';

            //$queryParams['order'] = 'date';
            $videoFeed = $youtube->paginateResults($queryParams, $pageToken);

        }
        foreach ($videoFeed['results'] as $searchResult) {
            if (isset($searchResult->id->videoId) || isset($searchResult->contentDetails->videoId)) {
                if (self::MODE_PLAYLIST == $requestType) {
                    $videoId = $searchResult->contentDetails->videoId;
                } else {
                    $videoId = $searchResult->id->videoId;
                }
                $videos[] = array(
                    'vId' => $videoId,
                    'url' => $searchResult->snippet->thumbnails->high->url,
                    'title' => $searchResult->snippet->title,
                    'titlelong' => $searchResult->snippet->description
                );
            }
        }
        return array('feed' => $videoFeed, 'videos'=>$videos);
    }

    public function getVideoThumbsAction()
    {
        echo json_encode(array('content' => $this->_getVideoThumbs()));
    }

    public function checkVideolinkAction()
    {
        $useVideolink = $this->_request->getParam('usewithVideolink');
        $pluginMapper = Application_Model_Mappers_PluginMapper::getInstance();
        $exsistVideolink = $pluginMapper->findByName('webbuilder');
        if ($exsistVideolink == null) {
            echo json_encode(array('pluginExcist' => 0, 'checkbox' => $useVideolink));
        } else {
            echo json_encode(array('pluginExcist' => 1, 'checkbox' => $useVideolink));
        }
    }

    public function getPlayerAction()
    {
        echo json_encode(array('content' => $this->_getPlayer()));
    }

    public function _getPlayer()
    {
        if ($this->_request->isPost()) {
            $videoId = $this->_request->getParam('videoId');
            $pageId = $this->_request->getParam('pageId');
            if ($videoId) {
                $youTube = new Youtube(array('key' => $this->_googleApiKey));
                $mapper = YoutubefeedMapper::getInstance();
                $сonfig = $mapper->selectProperties($pageId);
                if (!empty($сonfig)) {
                    $this->_view->playerWidth = $сonfig[0]->getPlayerWidth();
                    $this->_view->playerHeight = $сonfig[0]->getPlayerHeight();
                } else {
                    $this->_view->playerWidth = 280;
                    $this->_view->playerHeight = 200;
                }
                $entry = $youTube->getVideoInfo($videoId);
                $result = $this->_getVideoMetaData($entry);
                //$url = $videoId;
                //$url = preg_replace("/\/v\//", '/embed/', $url);
                $this->_view->url = $videoId;
                $this->_view->meta = $result;

                return $this->_view->render('player.phtml');
            }
        }
    }

    private function _getVideoMetaData($entry)
    {
        return array(
            'title' => isset($entry->snippet->title) ? $entry->snippet->title : '',
            'description' => isset($entry->snippet->description) ? $entry->snippet->description : '',
            'duration' => isset($entry->contentDetails->duration) ? $entry->contentDetails->duration : '',
            'views' => isset($entry->statistics->viewCount) ? $entry->statistics->viewCount : '0',
            'commentCount' => isset($entry->statistics->commentCount) ? $entry->statistics->commentCount : '0'
        );
    }

    public function _getFlashUrl($entry)
    {
        foreach ($entry->mediaGroup->content as $content) {
            if ($content->type === 'application/x-shockwave-flash') {
                return $content->url;
            }
        }

        return null;
    }

    public function showSettingsAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
            $mapper = YoutubefeedMapper::getInstance();
            $pageId = $this->_request->getParam('pageId');
            $сonfig = $mapper->selectProperties($pageId);
            $this->_view->pageId = $pageId;
            $this->_view->translator = $this->_translator;
            if (empty($сonfig)) {
                $сonfig = $mapper->selectProperties('0');
            }
            $this->_view->thumbsWidth = $сonfig[0]->getThumpWidth();
            $this->_view->thumbsHeight = $сonfig[0]->getThumbHeight();
            $this->_view->thumbsPerPage = $сonfig[0]->getThumbPerPage();
            $this->_view->usewithVideolink = $сonfig[0]->getUseVideolink();
            $this->_view->playerWidth = $сonfig[0]->getPlayerWidth();
            $this->_view->playerHeight = $сonfig[0]->getPlayerHeight();
            $this->_layout->content = $this->_view->render('settings.phtml');
            echo $this->_layout->render();
        }
    }

    public function applySettingsAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::YOUTUBEFEED_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('');
            }
            $thumbsWidth = trim($this->_request->getParam('thumbsWidth'));
            $thumbsHeight = trim($this->_request->getParam('thumbsHeight'));
            $thumbsPerPage = trim($this->_request->getParam('thumbsPerPage'));
            $useVideolink = trim($this->_request->getParam('usewithVideolink'));
            $playerWidth = trim($this->_request->getParam('playerWidth'));
            $playerHeight = trim($this->_request->getParam('playerHeight'));
            $pageId = $this->_request->getParam('pageId');
            $mapper = YoutubefeedMapper::getInstance();
            $model = new YoutubefeedModel();
            $model->setThumpWidth($thumbsWidth);
            $model->setThumbHeight($thumbsHeight);
            $model->setThumbPerPage($thumbsPerPage);
            $model->setPlayerWidth($playerWidth);
            $model->setPlayerHeight($playerHeight);
            $model->setUseVideolink($useVideolink);
            $model->setPageId($pageId);
            $mapper->save($model);
            $this->_responseHelper->success('Saved');
        }
    }

}

